/**************************************************************************//**
 * @file     main.c
 * @version  V1.00
 * $Revision: 2 $
 * $Date: 15/04/01 1:17p $
 * @brief    To show how to use MDU + EPWM to implement FOC and output SVPWM waveform.
 * @note
 * Copyright (C) 2015 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "NM1530.h"

#define PLLCON_SETTING  CLK_PLLCON_72MHz_HXT
#define PLL_CLOCK       72000000

#define USE_MDU     0
#define MDU         MDU0


/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/

#define C_Points    512

uint16_t    ui_2_time_count = 0, ui_5_time_count = 0, ui_10_time_count = 0;
uint16_t    sin_index1 = 0;     // Index of sine table address
int16_t     as16Ia[C_Points];   
int16_t     as16Vref1[C_Points], as16Vref2[C_Points], as16Vref3[C_Points];
uint16_t    au16SVPDATA0[C_Points], au16SVPDATA2[C_Points], au16SVPDATA4[C_Points];
uint16_t    au16SVPDATA0N[C_Points], au16SVPDATA2N[C_Points], au16SVPDATA4N[C_Points];
int32_t     temp_bldc_synduty = 20000; 

/* MDU internal data to set registers. */
int16_t     sin_table_value, cos_table_value, current_Ia, current_Ib, current_Ic;
int16_t     Vq_Amplitude = -32767, Vd_Amplitude = 0; //"MDU_Vq = Amplitude" in the program
uint16_t    C_Tpwm = 4096; //It will set "PWM0_PWMP0=C_Tpwm" in the program
int16_t     Id_CMD = 0, Iq_CMD = 100;
int32_t     KP_d = 60, KI_d = 20, KP_q = 80, KI_q = 20;
int16_t     PI_dFW = 50, PI_qFW = 150;
int16_t     PI_dMAX = 32767, PI_dMIN = -32768, PI_qMAX = 32000, PI_qMIN = -31000;
int16_t     I_dMAX = 32767, I_dMIN = -32768, I_qMAX = 31000, I_qMIN = -30000;

void SYS_Init(void);
void UART0_Init(void);
void MDU_Init(void);
void Simulate_SIN_Calculation(uint16_t sine_table_counter);
void MDU_Run_One_Cycle1(void);

/*---------------------------------------------------------------------------------------------------------*/
/* MAIN function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
int main(void)
{
    /*
        This sample code use MDU + EPWM to output SVPWM wave.
        Use EPWM0 ~ EPWM5 to be the output PWM pin.
    */

    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Init System, peripheral clock and multi-function I/O */
    SYS_Init();

    /* Init UART0 for printf */
    UART0_Init();

    /* Init MDU */
    MDU_Init();

    /* Set EPWM0 Timer clock divider select */
    EPWM0->PWMCON = (EPWM0->PWMCON & ~(EPWM_PWMCON_PWMDIV_Msk)) | (EPWM_CLK_DIV_2);

    /* Set EPWM0 pair 0 duty */
    EPWM0->PWM0 = 17999;
    EPWM0->PWM2 = 17999;
    EPWM0->PWM4 = 17999;

    /* Set EPWM0 Timer period */
    EPWM0->PWMP = 59999;

    /* Lock protected registers */
    SYS_LockReg();

    /* Enable all EPWM0 output channels */
    GPIO->PWMPOEN &= ~(GPIO_PWMPOEN_HZ_Odd0_Msk | GPIO_PWMPOEN_HZ_Even0_Msk);

    /* Enable MDU auto load */
    EPWM0->PWMCON |= EPWM_PWMCON_AUTOLD_Msk | 1;

    /* Set Dead-Zone */
    EPWM0->PDTC = EPWM_PDTC_DTEN0_Msk | EPWM_PDTC_DTEN2_Msk | EPWM_PDTC_DTEN4_Msk | 32;

    printf("+---------------------------------------------------+\n");
    printf("|            MDU Driver Sample Code                 |\n");
    printf("+---------------------------------------------------+\n");
    printf("  This sample code is to test MDU to output          \n");
    printf("  SVPWM wave                                         \n");

    printf("\nStart...\n");

    /* Start */
    EPWM0->PWMCON |= EPWM_PWMCON_PWMRUN_Msk;

    while(1)
    {
        for(sin_index1 = 0; sin_index1 < C_Points; sin_index1++)
        {
            MDU_Run_One_Cycle1();
            CLK_SysTickDelay(10000); // 100Hz per iteration
        }
    }

}


void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable Internal RC 22.1184MHz clock */
    CLK->PWRCON |= CLK_PWRCON_OSC22M_EN_Msk;

    /* Waiting for Internal RC clock ready */
    while(!(CLK->CLKSTATUS & CLK_CLKSTATUS_OSC22M_STB_Msk));

    /* Switch HCLK clock source to Internal RC and HCLK source divide 1 */
    CLK->CLKSEL0 &= ~CLK_CLKSEL0_HCLK_S_Msk;
    CLK->CLKSEL0 |= CLK_CLKSEL0_HCLK_S_HIRC;
    CLK->CLKDIV &= ~CLK_CLKDIV_HCLK_N_Msk;
    CLK->CLKDIV |= CLK_CLKDIV_HCLK(1);

    /* Enable external XTAL 12MHz clock */
    CLK->PWRCON |= CLK_PWRCON_XTL12M_EN_Msk;

    /* Waiting for external XTAL clock ready */
    while(!(CLK->CLKSTATUS & CLK_CLKSTATUS_XTL12M_STB_Msk));

    /* Set core clock as PLL_CLOCK from PLL */
    CLK->PLLCON = PLLCON_SETTING;
    while(!(CLK->CLKSTATUS & CLK_CLKSTATUS_PLL_STB_Msk));
    CLK->CLKSEL0 &= (~CLK_CLKSEL0_HCLK_S_Msk);
    CLK->CLKSEL0 |= CLK_CLKSEL0_HCLK_S_PLL;

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    //SystemCoreClockUpdate();
    PllClock        = PLL_CLOCK;            // PLL
    SystemCoreClock = PLL_CLOCK / 1;        // HCLK
    CyclesPerUs     = PLL_CLOCK / 1000000;  // For SYS_SysTickDelay()

    /* Enable UART module clock */
    CLK->APBCLK |= CLK_APBCLK_UART0_EN_Msk;

    /* Select UART module clock source */
    CLK->CLKSEL1 &= ~CLK_CLKSEL1_UART_S_Msk;
    CLK->CLKSEL1 |= CLK_CLKSEL1_UART_S_HXT;

    /* Enable MDU module clock */
    CLK->APBCLK |= CLK_APBCLK_MDU_EN_Msk;

    /* Enable EPWM0 module clock */
    CLK->APBCLK |= CLK_APBCLK_EPWM0_EN_Msk;

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Set P3 multi-function pins for UART0 RXD and TXD */
    SYS->P3_MFP &= ~(SYS_MFP_P30_Msk | SYS_MFP_P31_Msk);
    SYS->P3_MFP |= (SYS_MFP_P30_UART0_RXD | SYS_MFP_P31_UART0_TXD);

    /* Set P0 multi-function pins for EPWM Channel 0(PWM0_CH0) ~ channel 5(PWM0_CH5) */
    SYS->P0_MFP &= ~(SYS_MFP_P00_Msk | SYS_MFP_P01_Msk | SYS_MFP_P02_Msk | SYS_MFP_P03_Msk | SYS_MFP_P04_Msk | SYS_MFP_P05_Msk);
    SYS->P0_MFP |= (SYS_MFP_P00_EPWM0_CH0 | SYS_MFP_P01_EPWM0_CH1 | SYS_MFP_P02_EPWM0_CH2 | SYS_MFP_P03_EPWM0_CH3 | SYS_MFP_P04_EPWM0_CH4 | SYS_MFP_P05_EPWM0_CH5);

}

void UART0_Init()
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART IP */
    SYS->IPRSTC2 |=  SYS_IPRSTC2_UART0_RST_Msk;
    SYS->IPRSTC2 &= ~SYS_IPRSTC2_UART0_RST_Msk;

    /* Configure UART0 and set UART0 Baudrate */
    UART0->BAUD = UART_BAUD_MODE2 | UART_BAUD_MODE2_DIVIDER(__HXT, 115200);
    UART0->LCR = UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1;
}

void MDU_Init(void)
{
    MDUG_SELECT(MDUG, USE_MDU); // Select a register set to control MDU
    MDU->MDUCON = 0x0;          // Clear all MDU control registers
    MDU->MDUSTS = 0xEF;         // Clear all status

    MDU->Id_CMD = Id_CMD;       // Set Id command
    MDU->Iq_CMD = Iq_CMD;       // Set Iq command
    MDU->KP_d = KP_d;           // Set KP_d
    MDU->KI_d = KI_d;           // Set KI_d
    MDU->KP_q = KP_q;           // Set KP_q
    MDU->KI_q = KI_q;           // Set KI_q
    MDU->PI_dFW = PI_dFW;       // Set d-axis feedfoward item
    MDU->PI_qFW = PI_qFW;       // Set q-axis feedforwad item
    MDU->PI_dMAX = PI_dMAX;     // Set PI limit
    MDU->PI_dMIN = PI_dMIN;
    MDU->PI_qMAX = PI_qMAX;
    MDU->PI_qMIN = PI_qMIN;
    MDU->I_dMAX = I_dMAX;       // Set I-controller limit
    MDU->I_dMIN = I_dMIN;
    MDU->I_qMAX = I_qMAX;
    MDU->I_qMIN = I_qMIN;
    MDU->PWMNORM = C_Tpwm;
    MDU->PWMMAX = C_Tpwm - 0;
    MDU->PWMMIN = 0;

    MDU->UI_d = 0;      // Set initial value of integrator
    MDU->UI_q = 0;      // Set initial value of integrator

//--Set All function block except Clark to Auto-mode-------------------
    MDU->MDUCON = 0x00007E00;       // Set all AUTO bits to high

    MDU_DISABLE_AUTO_CLEAR(MDU);    // Do not auto clear flag for test program

//--- Set PWM0_PWMP0 for MDU use in SVPWM block--------
    EPWM0->PWMP = C_Tpwm;
}

//*****Simulate_SIN_Calculation()****************************************************************
// This function use 3 sin table to simulate Ia, Ib and Ic
//*********************************************************************
void Simulate_SIN_Calculation(uint16_t sine_table_counter) //run in the PWM Subroutine
{
    int16_t temp_bldc_synduty_U;
    int16_t temp_bldc_synduty_V;
    int16_t temp_bldc_synduty_W;
    uint32_t temp_120_sine_table_counter, sine_counter;
    uint32_t point = C_Points;

    /* 512 Q15 sin table for Park transform use */
    static const int16_t sine_a_table[C_Points] =           
    {
        0,    402,    804,   1206,   1608,   2009,   2410,   2811,
        3212,   3612,   4011,   4410,   4808,   5205,   5602,   5998,
        6393,   6786,   7179,   7571,   7962,   8351,   8739,   9126,
        9512,   9896,  10278,  10659,  11039,  11417,  11793,  12167,
        12539,  12910,  13279,  13645,  14010,  14372,  14732,  15090,
        15446,  15800,  16151,  16499,  16846,  17189,  17530,  17869,
        18204,  18537,  18868,  19195,  19519,  19841,  20159,  20475,
        20787,  21096,  21403,  21705,  22005,  22301,  22594,  22884,
        23170,  23452,  23731,  24007,  24279,  24547,  24811,  25072,
        25329,  25582,  25832,  26077,  26319,  26556,  26790,  27019,
        27245,  27466,  27683,  27896,  28105,  28310,  28510,  28706,
        28898,  29085,  29268,  29447,  29621,  29791,  29956,  30117,
        30273,  30424,  30571,  30714,  30852,  30985,  31113,  31237,
        31356,  31470,  31580,  31685,  31785,  31880,  31971,  32057,
        32137,  32213,  32285,  32351,  32412,  32469,  32521,  32567,
        32609,  32646,  32678,  32705,  32728,  32745,  32757,  32765,
        32767,  32765,  32757,  32745,  32728,  32705,  32678,  32646,
        32609,  32567,  32521,  32469,  32412,  32351,  32285,  32213,
        32137,  32057,  31971,  31880,  31785,  31685,  31580,  31470,
        31356,  31237,  31113,  30985,  30852,  30714,  30571,  30424,
        30273,  30117,  29956,  29791,  29621,  29447,  29268,  29085,
        28898,  28706,  28510,  28310,  28105,  27896,  27683,  27466,
        27245,  27019,  26790,  26556,  26319,  26077,  25832,  25582,
        25329,  25072,  24811,  24547,  24279,  24007,  23731,  23452,
        23170,  22884,  22594,  22301,  22005,  21705,  21403,  21096,
        20787,  20475,  20159,  19841,  19519,  19195,  18868,  18537,
        18204,  17869,  17530,  17189,  16846,  16499,  16151,  15800,
        15446,  15090,  14732,  14372,  14010,  13645,  13279,  12910,
        12539,  12167,  11793,  11417,  11039,  10659,  10278,   9896,
        9512,   9126,   8739,   8351,   7962,   7571,   7179,   6786,
        6393,   5998,   5602,   5205,   4808,   4410,   4011,   3612,
        3212,   2811,   2410,   2009,   1608,   1206,    804,    402,
        0,   -402,   -804,  -1206,  -1608,  -2009,  -2410,  -2811,
        -3212,  -3612,  -4011,  -4410,  -4808,  -5205,  -5602,  -5998,
        -6393,  -6786,  -7179,  -7571,  -7962,  -8351,  -8739,  -9126,
        -9512,  -9896, -10278, -10659, -11039, -11417, -11793, -12167,
        -12539, -12910, -13279, -13645, -14010, -14372, -14732, -15090,
        -15446, -15800, -16151, -16499, -16846, -17189, -17530, -17869,
        -18204, -18537, -18868, -19195, -19519, -19841, -20159, -20475,
        -20787, -21096, -21403, -21705, -22005, -22301, -22594, -22884,
        -23170, -23452, -23731, -24007, -24279, -24547, -24811, -25072,
        -25329, -25582, -25832, -26077, -26319, -26556, -26790, -27019,
        -27245, -27466, -27683, -27896, -28105, -28310, -28510, -28706,
        -28898, -29085, -29268, -29447, -29621, -29791, -29956, -30117,
        -30273, -30424, -30571, -30714, -30852, -30985, -31113, -31237,
        -31356, -31470, -31580, -31685, -31785, -31880, -31971, -32057,
        -32137, -32213, -32285, -32351, -32412, -32469, -32521, -32567,
        -32609, -32646, -32678, -32705, -32728, -32745, -32757, -32765,
        -32767, -32765, -32757, -32745, -32728, -32705, -32678, -32646,
        -32609, -32567, -32521, -32469, -32412, -32351, -32285, -32213,
        -32137, -32057, -31971, -31880, -31785, -31685, -31580, -31470,
        -31356, -31237, -31113, -30985, -30852, -30714, -30571, -30424,
        -30273, -30117, -29956, -29791, -29621, -29447, -29268, -29085,
        -28898, -28706, -28510, -28310, -28105, -27896, -27683, -27466,
        -27245, -27019, -26790, -26556, -26319, -26077, -25832, -25582,
        -25329, -25072, -24811, -24547, -24279, -24007, -23731, -23452,
        -23170, -22884, -22594, -22301, -22005, -21705, -21403, -21096,
        -20787, -20475, -20159, -19841, -19519, -19195, -18868, -18537,
        -18204, -17869, -17530, -17189, -16846, -16499, -16151, -15800,
        -15446, -15090, -14732, -14372, -14010, -13645, -13279, -12910,
        -12539, -12167, -11793, -11417, -11039, -10659, -10278,  -9896,
        -9512,  -9126,  -8739,  -8351,  -7962,  -7571,  -7179,  -6786,
        -6393,  -5998,  -5602,  -5205,  -4808,  -4410,  -4011,  -3612,
        -3212,  -2811,  -2410,  -2009,  -1608,  -1206,   -804,   -402
    };

    /* 512 Q15 sin table for Park transform use */
    static const int16_t sine_b_table[C_Points] =           
    {
        -28377, -28576, -28771, -28961, -29147, -29328, -29505, -29678,
        -29846, -30010, -30169, -30324, -30474, -30619, -30760, -30896,
        -31028, -31155, -31277, -31395, -31507, -31616, -31719, -31817,
        -31911, -32000, -32084, -32163, -32238, -32307, -32372, -32432,
        -32487, -32537, -32582, -32622, -32657, -32688, -32713, -32734,
        -32749, -32760, -32766, -32767, -32763, -32754, -32740, -32721,
        -32697, -32668, -32634, -32596, -32552, -32504, -32451, -32392,
        -32329, -32261, -32189, -32111, -32028, -31941, -31849, -31752,
        -31650, -31544, -31433, -31317, -31196, -31071, -30941, -30806,
        -30667, -30523, -30374, -30221, -30064, -29901, -29735, -29563,
        -29388, -29208, -29023, -28834, -28641, -28444, -28242, -28036,
        -27826, -27611, -27393, -27170, -26943, -26712, -26478, -26239,
        -25996, -25749, -25498, -25244, -24986, -24724, -24458, -24189,
        -23915, -23639, -23359, -23075, -22788, -22497, -22203, -21905,
        -21605, -21301, -20994, -20683, -20370, -20053, -19734, -19411,
        -19086, -18758, -18427, -18093, -17756, -17417, -17075, -16730,
        -16383, -16034, -15682, -15328, -14971, -14613, -14252, -13888,
        -13523, -13156, -12787, -12415, -12042, -11668, -11291, -10913,
        -10533, -10151,  -9768,  -9383,  -8997,  -8610,  -8222,  -7832,
        -7441,  -7048,  -6655,  -6261,  -5866,  -5470,  -5073,  -4675,
        -4277,  -3878,  -3478,  -3078,  -2678,  -2277,  -1875,  -1474,
        -1072,   -670,   -268,    134,    536,    938,   1340,   1742,
        2143,   2544,   2945,   3345,   3745,   4144,   4543,   4940,
        5338,   5734,   6129,   6524,   6917,   7310,   7701,   8092,
        8481,   8868,   9255,   9640,  10024,  10406,  10786,  11165,
        11542,  11918,  12291,  12663,  13033,  13401,  13767,  14131,
        14492,  14852,  15209,  15564,  15917,  16267,  16615,  16960,
        17303,  17643,  17981,  18316,  18648,  18977,  19303,  19627,
        19947,  20265,  20579,  20891,  21199,  21504,  21806,  22104,
        22399,  22691,  22979,  23264,  23546,  23824,  24098,  24369,
        24636,  24899,  25158,  25414,  25666,  25914,  26158,  26398,
        26635,  26867,  27095,  27319,  27539,  27755,  27966,  28174,
        28377,  28576,  28771,  28961,  29147,  29328,  29505,  29678,
        29846,  30010,  30169,  30324,  30474,  30619,  30760,  30896,
        31028,  31155,  31277,  31395,  31507,  31616,  31719,  31817,
        31911,  32000,  32084,  32163,  32238,  32307,  32372,  32432,
        32487,  32537,  32582,  32622,  32657,  32688,  32713,  32734,
        32749,  32760,  32766,  32767,  32763,  32754,  32740,  32721,
        32697,  32668,  32634,  32596,  32552,  32504,  32451,  32392,
        32329,  32261,  32189,  32111,  32028,  31941,  31849,  31752,
        31650,  31544,  31433,  31317,  31196,  31071,  30941,  30806,
        30667,  30523,  30374,  30221,  30064,  29901,  29735,  29563,
        29388,  29208,  29023,  28834,  28641,  28444,  28242,  28036,
        27826,  27611,  27393,  27170,  26943,  26712,  26478,  26239,
        25996,  25749,  25498,  25244,  24986,  24724,  24458,  24189,
        23915,  23639,  23359,  23075,  22788,  22497,  22203,  21905,
        21605,  21301,  20994,  20683,  20370,  20053,  19734,  19411,
        19086,  18758,  18427,  18093,  17756,  17417,  17075,  16730,
        16383,  16034,  15682,  15328,  14971,  14613,  14252,  13888,
        13523,  13156,  12787,  12415,  12042,  11668,  11291,  10913,
        10533,  10151,   9768,   9383,   8997,   8610,   8222,   7832,
        7441,   7048,   6655,   6261,   5866,   5470,   5073,   4675,
        4277,   3878,   3478,   3078,   2678,   2277,   1875,   1474,
        1072,    670,    268,   -134,   -536,   -938,  -1340,  -1742,
        -2143,  -2544,  -2945,  -3345,  -3745,  -4144,  -4543,  -4940,
        -5338,  -5734,  -6129,  -6524,  -6917,  -7310,  -7701,  -8092,
        -8481,  -8868,  -9255,  -9640, -10024, -10406, -10786, -11165,
        -11542, -11918, -12291, -12663, -13033, -13401, -13767, -14131,
        -14492, -14852, -15209, -15564, -15917, -16267, -16615, -16960,
        -17303, -17643, -17981, -18316, -18648, -18977, -19303, -19627,
        -19947, -20265, -20579, -20891, -21199, -21504, -21806, -22104,
        -22399, -22691, -22979, -23264, -23546, -23824, -24098, -24369,
        -24636, -24899, -25158, -25414, -25666, -25914, -26158, -26398,
        -26635, -26867, -27095, -27319, -27539, -27755, -27966, -28174
    };

    /* 512 Q15 sin table for Park transform use */
    static const int16_t sine_c_table[C_Points] =           
    {
        28377,  28174,  27967,  27755,  27539,  27319,  27095,  26867,
        26634,  26398,  26158,  25914,  25666,  25414,  25158,  24898,
        24635,  24369,  24098,  23824,  23545,  23265,  22980,  22691,
        22399,  22104,  21806,  21504,  21199,  20890,  20579,  20265,
        19948,  19627,  19303,  18977,  18647,  18316,  17981,  17644,
        17303,  16960,  16615,  16268,  15917,  15565,  15210,  14852,
        14493,  14131,  13766,  13401,  13033,  12663,  12292,  11917,
        11542,  11165,  10786,  10406,  10023,   9640,   9255,   8868,
        8480,   8092,   7702,   7310,   6917,   6524,   6130,   5734,
        5338,   4941,   4542,   4144,   3745,   3345,   2945,   2544,
        2143,   1742,   1340,    938,    536,    134,   -268,   -670,
        -1072,  -1474,  -1875,  -2277,  -2678,  -3079,  -3478,  -3878,
        -4277,  -4675,  -5073,  -5470,  -5866,  -6261,  -6655,  -7048,
        -7441,  -7831,  -8221,  -8610,  -8997,  -9383,  -9768, -10152,
        -10532, -10912, -11291, -11668, -12042, -12416, -12787, -13156,
        -13523, -13888, -14251, -14612, -14972, -15328, -15682, -16035,
        -16384, -16731, -17075, -17417, -17757, -18092, -18426, -18758,
        -19086, -19411, -19734, -20054, -20370, -20683, -20994, -21300,
        -21604, -21906, -22203, -22497, -22788, -23075, -23358, -23638,
        -23915, -24189, -24458, -24724, -24986, -25244, -25498, -25749,
        -25996, -26239, -26478, -26713, -26943, -27170, -27393, -27611,
        -27826, -28036, -28242, -28444, -28641, -28834, -29023, -29208,
        -29388, -29563, -29735, -29901, -30064, -30221, -30375, -30522,
        -30667, -30806, -30940, -31071, -31196, -31317, -31432, -31544,
        -31651, -31752, -31849, -31941, -32029, -32111, -32189, -32261,
        -32329, -32393, -32450, -32504, -32552, -32596, -32635, -32668,
        -32696, -32721, -32739, -32753, -32763, -32766, -32766, -32760,
        -32749, -32733, -32713, -32688, -32658, -32622, -32582, -32537,
        -32486, -32432, -32372, -32308, -32238, -32163, -32084, -32000,
        -31911, -31817, -31718, -31615, -31508, -31395, -31277, -31155,
        -31029, -30897, -30760, -30619, -30474, -30324, -30169, -30010,
        -29847, -29678, -29505, -29328, -29147, -28961, -28770, -28576,
        -28377, -28174, -27967, -27755, -27539, -27319, -27095, -26867,
        -26634, -26398, -26158, -25914, -25666, -25414, -25158, -24898,
        -24635, -24369, -24098, -23824, -23545, -23265, -22980, -22691,
        -22399, -22104, -21806, -21504, -21199, -20890, -20579, -20265,
        -19948, -19627, -19303, -18977, -18647, -18316, -17981, -17644,
        -17303, -16960, -16615, -16268, -15917, -15565, -15210, -14852,
        -14493, -14131, -13766, -13401, -13033, -12663, -12292, -11917,
        -11542, -11165, -10786, -10406, -10023,  -9640,  -9255,  -8868,
        -8480,  -8092,  -7702,  -7310,  -6917,  -6524,  -6130,  -5734,
        -5338,  -4941,  -4542,  -4144,  -3745,  -3345,  -2945,  -2544,
        -2143,  -1742,  -1340,   -938,   -536,   -134,    268,    670,
        1072,   1474,   1875,   2277,   2678,   3079,   3478,   3878,
        4277,   4675,   5073,   5470,   5866,   6261,   6655,   7048,
        7441,   7831,   8221,   8610,   8997,   9383,   9768,  10152,
        10532,  10912,  11291,  11668,  12042,  12416,  12787,  13156,
        13523,  13888,  14251,  14612,  14972,  15328,  15682,  16035,
        16384,  16731,  17075,  17417,  17757,  18092,  18426,  18758,
        19086,  19411,  19734,  20054,  20370,  20683,  20994,  21300,
        21604,  21906,  22203,  22497,  22788,  23075,  23358,  23638,
        23915,  24189,  24458,  24724,  24986,  25244,  25498,  25749,
        25996,  26239,  26478,  26713,  26943,  27170,  27393,  27611,
        27826,  28036,  28242,  28444,  28641,  28834,  29023,  29208,
        29388,  29563,  29735,  29901,  30064,  30221,  30375,  30522,
        30667,  30806,  30940,  31071,  31196,  31317,  31432,  31544,
        31651,  31752,  31849,  31941,  32029,  32111,  32189,  32261,
        32329,  32393,  32450,  32504,  32552,  32596,  32635,  32668,
        32696,  32721,  32739,  32753,  32763,  32766,  32766,  32760,
        32749,  32733,  32713,  32688,  32658,  32622,  32582,  32537,
        32486,  32432,  32372,  32308,  32238,  32163,  32084,  32000,
        31911,  31817,  31718,  31615,  31508,  31395,  31277,  31155,
        31029,  30897,  30760,  30619,  30474,  30324,  30169,  30010,
        29847,  29678,  29505,  29328,  29147,  28961,  28770,  28576
    };


    temp_120_sine_table_counter =  sine_table_counter;
    sine_counter = sine_table_counter;

    temp_bldc_synduty_U = (signed short int)((temp_bldc_synduty * sine_a_table[temp_120_sine_table_counter]) >> 15);
    temp_bldc_synduty_V = (signed short int)((temp_bldc_synduty * sine_b_table[temp_120_sine_table_counter]) >> 15);
    temp_bldc_synduty_W = (signed short int)((temp_bldc_synduty * sine_c_table[temp_120_sine_table_counter]) >> 15);

    current_Ia = temp_bldc_synduty_U;       //load pwm duty to pwm ouptput pin
    current_Ib = temp_bldc_synduty_V;       //load pwm duty to pwm ouptput pin
    current_Ic = temp_bldc_synduty_W;       //load pwm duty to pwm ouptput pin

    //----SIN/COS table value---------
    sin_table_value = (signed short int) sine_a_table[sine_counter];        //Q15 sin

    sine_counter = sine_counter + point / 4;
    if(sine_counter >= point)
        sine_counter = sine_counter - point;

    cos_table_value = (signed short int) sine_a_table[sine_counter];        //Q15 cos
}

/*******MDU_Run_One_Cycle1(void)**********************************************************************
 * Before execute this function, the registers content of Ia, Ib & Ic must be preset in Q15 format.
 *****************************************************************************************************/
void MDU_Run_One_Cycle1(void)
{

//---Prepare Ia/Ib/Ic for MDU---------
    Simulate_SIN_Calculation(sin_index1);

//---Simulate calulation for MDU_SIN & MDU_COS-------
    MDU->SIN = sin_table_value;
    MDU->COS = cos_table_value;

    MDU->Ia = current_Ia;
    MDU->Ib = current_Ib;
    MDU->Ic = current_Ic;

//---Start Clark transform---------------------------------
    MDU_START(MDU, MDU_MDUCON_CKSTR_Msk);                        // Start Clark transform
    while(MDU_GET_FLAG(MDU, MDU_MDUSTS_CKCPF_Msk) == FALSE);    // Wait for Clark transform complete
    MDU_CLEAR_INT(MDU, MDU_MDUSTS_CKCPF_Msk);

//---Start Park tranform---------------------------------------
    MDU_START(MDU, MDU_MDUCON_PKSTR_Msk);                        // Start Park transform
    while(MDU_GET_FLAG(MDU, MDU_MDUSTS_PKCPF_Msk) == FALSE);    // Wait for Park transform complete
    MDU_CLEAR_INT(MDU, MDU_MDUSTS_PKCPF_Msk);

//---Start PI controller-----------------------------------------
    MDU_START(MDU, MDU_MDUCON_PIDSTR_Msk | MDU_MDUCON_PIQSTR_Msk);  // Start PI control
    while(MDU_GET_FLAG(MDU, MDU_MDUSTS_PI_dCPF_Msk | MDU_MDUSTS_PI_qCPF_Msk) == FALSE);  // Wait for Clark transform complete
    MDU_CLEAR_INT(MDU, MDU_MDUSTS_PI_dCPF_Msk | MDU_MDUSTS_PI_qCPF_Msk);

//---Start Inverse Park--------------------------------
    MDU_START(MDU, MDU_MDUCON_INVPKSTR_Msk);                        // Start Inverse Park transform
    while(MDU_GET_FLAG(MDU, MDU_MDUSTS_INVPKCPF_Msk) == FALSE);    // Wait for Inverse Park transform complete
    MDU_CLEAR_INT(MDU, MDU_MDUSTS_INVPKCPF_Msk);

//---Start Modified Inverse Clark--------------------------------
    //MDU_ENABLE_SVPWM(MDU);  // modified inverse clark transform
    //MDU_DISABLE_SVPWM(MDU); // inverse clark transform

    MDU_START(MDU, MDU_MDUCON_INVCKSTR_Msk);                        // Start Inverse Clark transform
    while(MDU_GET_FLAG(MDU, MDU_MDUSTS_INVCKCPF_Msk) == FALSE);    // Wait for Inverse Clark transform complete
    MDU_CLEAR_INT(MDU, MDU_MDUSTS_INVCKCPF_Msk);

//---Start SVPWM block--------------------------------
    MDU_START(MDU, MDU_MDUCON_SVPWMSTR_Msk);                        // Start to do SVPWM
    while(MDU_GET_FLAG(MDU, MDU_MDUSTS_SVPWMCPF_Msk) == FALSE);    // Wait for SVPWM complete
    MDU_CLEAR_INT(MDU, MDU_MDUSTS_SVPWMCPF_Msk);

//---Save the internal data registers in the array buffer------------------
    as16Ia[sin_index1] = MDU->Ia;
    au16SVPDATA0[sin_index1] = MDU->SVPDATA0;
    au16SVPDATA2[sin_index1] = MDU->SVPDATA2;
    au16SVPDATA4[sin_index1] = MDU->SVPDATA4;
    au16SVPDATA0N[sin_index1] = MDU->SVPDATA0N;
    au16SVPDATA2N[sin_index1] = MDU->SVPDATA2N;
    au16SVPDATA4N[sin_index1] = MDU->SVPDATA4N;
    as16Vref1[sin_index1] = MDU->Vref1;     //PMDU_Iq
    as16Vref2[sin_index1] = MDU->Vref2;     //MDU_Vd
    as16Vref3[sin_index1] = MDU->Vref3;     //PMDU_Vq

//----------------------------------------------------------------------------------------------------

}


/*** (C) COPYRIGHT 2015 Nuvoton Technology Corp. ***/
